"""
notifications helper
"""
import requests

from django.contrib.auth.models import User

from allianceauth.notifications import notify
from allianceauth.services.hooks import get_extension_logger

from buybackprogram.app_settings import (
    aa_discordnotify_active,
    allianceauth_discordbot_active,
)

logger = get_extension_logger(__name__)


def send_aa_discordbot_notification(user, message):
    # If discordproxy app is not active we will check if aa-discordbot is active
    if allianceauth_discordbot_active():
        from aadiscordbot.tasks import send_message
        from discord import Embed

        embed = Embed(
            description=message["description"],
            title=message["title"],
            color=message["color"],
        )

        embed.set_footer(text=message["footer"])

        embed.add_field(name="Value", value=message["value"], inline=True)
        embed.add_field(name="Assigned to", value=message["assigned_to"], inline=True)
        embed.add_field(
            name="Assigned from", value=message["assigned_from"], inline=True
        )

        send_message(user_pk=user, embed=embed)

        logger.debug("Sent discord DM to user %s" % user)
    else:
        logger.debug(
            "No discord notification modules active. Will not send user notifications"
        )


def send_aa_discordbot_channel_notification(channel_id, message):
    # If discordproxy app is not active we will check if aa-discordbot is active
    if allianceauth_discordbot_active():
        import aadiscordbot.tasks

        aadiscordbot.tasks.send_channel_message_by_discord_id.delay(
            channel_id, message, embed=True
        )

        logger.debug("Sent notification to channel %s" % channel_id)
    else:
        logger.debug(
            "No discord notification modules active. Will not send user notifications"
        )


def send_user_notification(user: User, level: str, message: dict) -> None:

    # Send AA text notification
    notify(
        user=user,
        title=message["title"],
        level=level,
        message=message["description"],
    )

    if not aa_discordnotify_active():
        # Check if the discordproxy module is active. We will use it as our priority app for notifications
        try:

            from discordproxy.client import DiscordClient
            from discordproxy.discord_api_pb2 import Embed
            from discordproxy.exceptions import (
                DiscordProxyException,
                DiscordProxyGrpcError,
            )

            logger.debug("User has a active discord account")

            client = DiscordClient()

            fields = []

            fields.append(
                Embed.Field(name="Value", value=message["value"], inline=True)
            )
            fields.append(
                Embed.Field(
                    name="Assigned to", value=message["assigned_to"], inline=True
                )
            )
            fields.append(
                Embed.Field(
                    name="Assigned from", value=message["assigned_from"], inline=True
                )
            )

            embed = Embed(
                description=message["description"],
                title=message["title"],
                color=message["color"],
                footer=Embed.Footer(text=message["footer"]),
                fields=fields,
                author=Embed.Author(name="AA Buyback Program"),
            )

            if user.discord.uid:
                try:
                    logger.debug(
                        "Sending notification for discord user %s" % user.discord.uid
                    )
                    client.create_direct_message(user_id=user.discord.uid, embed=embed)
                except DiscordProxyGrpcError:
                    logger.debug(
                        "Discordprox is installed but not running, failed to send message. Attempting to send via aa-discordbot instead."
                    )
                    send_aa_discordbot_notification(user.pk, message)

                except DiscordProxyException as ex:
                    logger.error(
                        "An error occured when trying to create a message: %s" % ex
                    )
            else:
                logger.error(
                    "User has no active discord accounts on AUTH, passing DM message."
                )

        except ModuleNotFoundError:
            send_aa_discordbot_notification(user.pk, message)
    else:
        logger.debug(
            "Aadiscordnotify is already active, passing notification sending to prevent multiple notifications"
        )


def send_message_to_discord_channel(
    webhook, message: dict, embed: bool = False
) -> None:

    logger.debug("Notifications: Starting to parse channel webhook notification")

    url = webhook  # webhook url, from here: https://i.imgur.com/f9XnAew.png

    # for all params, see https://discordapp.com/developers/docs/resources/webhook#execute-webhook
    data = {}

    # leave this out if you dont want an embed
    # for all params, see https://discordapp.com/developers/docs/resources/channel#embed-object
    data["embeds"] = [
        {
            "description": message["description"],
            "title": message["title"],
            "color": message["color"],
            "footer": {"text": message["footer"]},
            "fields": [
                {"name": "Value", "value": message["value"], "inline": True},
                {
                    "name": "Assigned to",
                    "value": message["assigned_to"],
                    "inline": True,
                },
                {
                    "name": "Assigned from",
                    "value": message["assigned_from"],
                    "inline": True,
                },
            ],
        }
    ]

    result = requests.post(url, json=data)

    try:
        result.raise_for_status()
    except requests.exceptions.HTTPError as err:
        print(err)
    else:
        print("Payload delivered successfully, code {}.".format(result.status_code))



# Generated by Django 3.2.10 on 2022-04-06 15:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("buybackprogram", "0004_auto_20220323_1331"),
    ]

    operations = [
        migrations.AddField(
            model_name="program",
            name="compression_price_dencity_modifier",
            field=models.BooleanField(
                default=False,
                help_text="Should we apply price density calculations for items that can be compressed such as ore and ice. If set to False price density tax is not applied on any items that can be compressed.",
                verbose_name="Price density modifier for compressable items",
            ),
        ),
    ]



# Generated by Django 3.2.10 on 2022-03-23 13:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("buybackprogram", "0003_statics_performance_increase"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="program",
            name="location",
        ),
        migrations.AddField(
            model_name="program",
            name="location",
            field=models.ManyToManyField(
                help_text="The location where contracts should be created at.",
                related_name="_buybackprogram_program_location_+",
                to="buybackprogram.Location",
            ),
        ),
    ]



from django.contrib.auth.decorators import login_required, permission_required
from django.db import transaction
from django.http import HttpResponseRedirect
from django.shortcuts import redirect, render
from django.urls import reverse
from django.utils.html import format_html
from django.utils.translation import gettext_lazy
from esi.decorators import token_required

from allianceauth.authentication.models import CharacterOwnership
from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo
from allianceauth.services.hooks import get_extension_logger

from buybackprogram.forms import LocationForm, ProgramForm
from buybackprogram.models import Location, Owner, Program
from buybackprogram.utils import messages_plus

logger = get_extension_logger(__name__)


@login_required
@permission_required("buybackprogram.manage_programs")
@token_required(
    scopes=[
        "esi-contracts.read_character_contracts.v1",
        "esi-contracts.read_corporation_contracts.v1",
    ]
)
def setup(request, token):
    success = True
    token_char = EveCharacter.objects.get(character_id=token.character_id)

    try:
        owned_char = CharacterOwnership.objects.get(
            user=request.user, character=token_char
        )
    except CharacterOwnership.DoesNotExist:
        messages_plus.error(
            request,
            format_html(
                gettext_lazy(
                    "You can only use your main or alt characters "
                    "to add corporations. "
                    "However, character %s is neither. "
                )
                % format_html("<strong>{}</strong>", token_char.character_name)
            ),
        )
        success = False

    if success:
        try:
            corporation = EveCorporationInfo.objects.get(
                corporation_id=token_char.corporation_id
            )
        except EveCorporationInfo.DoesNotExist:
            corporation = EveCorporationInfo.objects.create_corporation(
                token_char.corporation_id
            )

        with transaction.atomic():
            owner, _ = Owner.objects.update_or_create(
                corporation=corporation, character=owned_char, user=request.user
            )

            owner.save()

        messages_plus.info(
            request,
            format_html(
                gettext_lazy(
                    "%(corporation)s has been added with %(character)s "
                    "as program manager. "
                )
                % {
                    "corporation": format_html(
                        "<strong>{}</strong>", owner.corporation
                    ),
                    "character": format_html(
                        "<strong>{}</strong>", owner.character.character.character_name
                    ),
                }
            ),
        )

    return redirect("buybackprogram:index")


@login_required
@permission_required("buybackprogram.manage_programs")
def program_add(request):
    form = ProgramForm(request.POST or None, user=request.user)

    if request.POST and form.is_valid():

        form = ProgramForm(request.POST, user=request.user)

        new_program = form.save()

        messages_plus.success(
            request,
            format_html(
                gettext_lazy("New program created at %(location)s")
                % {
                    "location": format_html(
                        "<strong>{}</strong>", new_program.location
                    ),
                }
            ),
        )

        return HttpResponseRedirect(reverse("buybackprogram:index"))

    return render(request, "buybackprogram/program_add.html", {"form": form})


@login_required
@permission_required("buybackprogram.manage_programs")
def program_edit(request, program_pk):

    instance = Program.objects.get(pk=program_pk)

    if request.method == "POST":
        form = ProgramForm(request.POST, user=request.user)

        logger.debug(
            "Received POST request containing update program form, is valid: %s"
            % form.is_valid()
        )

        if form.is_valid():

            form = ProgramForm(request.POST, instance=instance, user=request.user)

            updated_program = form.save()

            if updated_program.name:
                messages_plus.success(
                    request,
                    format_html(
                        gettext_lazy("Program %(name)s updated")
                        % {
                            "name": format_html(
                                "<strong>{}</strong>", updated_program.name
                            ),
                        }
                    ),
                )
            else:
                messages_plus.success(
                    request,
                    format_html(gettext_lazy("Program updated")),
                )

            return HttpResponseRedirect(reverse("buybackprogram:index"))

    else:

        form = ProgramForm(instance=instance, user=request.user)

    context = {
        "program": instance,
        "form": form,
    }

    return render(request, "buybackprogram/program_edit.html", context)


@login_required
@permission_required("buybackprogram.manage_programs")
def location_add(request):

    locations = Location.objects.all()

    if request.method != "POST":
        form = LocationForm(user=request.user)
    else:
        form = LocationForm(
            request.POST, value=int(request.POST["eve_solar_system"]), user=request.user
        )

        if form.is_valid():
            eve_solar_system = form.cleaned_data["eve_solar_system"]
            name = form.cleaned_data["name"]
            structure_id = form.cleaned_data["structure_id"] or None

            owner = Owner.objects.filter(user=request.user).first()

            created = Location.objects.update_or_create(
                eve_solar_system=eve_solar_system,
                name=name,
                structure_id=structure_id,
                owner=owner,
                defaults={
                    "eve_solar_system": eve_solar_system,
                    "name": name,
                    "structure_id": structure_id,
                    "owner": owner,
                },
            )

            if created:
                messages_plus.success(
                    request,
                    format_html(
                        "Added location for <strong>{}</strong> in system <strong>{}</strong>",
                        name,
                        eve_solar_system,
                    ),
                )

            return HttpResponseRedirect(request.path_info)

    context = {
        "locations": locations,
        "form": form,
    }

    return render(request, "buybackprogram/location_add.html", context)


@login_required
@permission_required("buybackprogram.manage_programs")
def location_remove(request, location_pk):

    location = Location.objects.get(pk=location_pk)

    if location.owner.user == request.user:
        location.delete()

        messages_plus.warning(
            request,
            format_html(
                gettext_lazy(
                    "Deleted location %(location)s. Any programs affiliated with this location has been removed as well."
                )
                % {
                    "location": format_html(
                        "<strong>{}</strong>", location.eve_solar_system
                    ),
                }
            ),
        )

    else:
        messages_plus.error(
            request,
            format_html(
                gettext_lazy(
                    "You did not create this location and thus you can't delete it."
                )
            ),
        )

    return redirect("/buybackprogram/location_add")


@login_required
@permission_required("buybackprogram.manage_programs")
def program_remove(request, program_pk):

    program = Program.objects.get(pk=program_pk)

    if program.owner.user == request.user:
        program.delete()

        messages_plus.warning(
            request,
            format_html(
                gettext_lazy("Deleted program %(name)s")
                % {
                    "name": format_html("<strong>{}</strong>", program.name),
                }
            ),
        )

    else:
        messages_plus.error(
            request,
            format_html(
                gettext_lazy(
                    "You do not own this program and thus you can't delete it."
                )
            ),
        )

    return redirect("buybackprogram:index")
